// Copyright (c) 2010 Luis Semp
// http://www.luissempe.com
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.

#include "Persistent.h"

void Persistent::Load(const char* xml)
{
	TiXmlDocument document;
	document.Parse(xml);

	Load(document.RootElement()->FirstChildElement());

	document.Clear();

}

void Persistent::LoadFromFile(const char* filename)
{
	TiXmlDocument document;
	if ( document.LoadFile(filename) )
	{
		Load(document.RootElement()->FirstChildElement());
		document.Clear();
	}
}

#define CASE(primType, from) case PT_##primType: *static_cast<primType*>(data->m_memberPtr) = static_cast<primType>( from ); break;
#define CASE2(primType, dataType, from) case primType: *static_cast<dataType*>(data->m_memberPtr) = static_cast<dataType>( from ); break;

void Persistent::Load(TiXmlNode* root)
{
	if ( !mRegistered )
	{
		Register();
		mRegistered = true;
	}

	PersistentTypeMap& typeMap = GetTypeMap();

	TiXmlElement* element = root->ToElement();
	if ( element )
	{
		const char* key = element->Value();
		TiXmlNode* childNode = element->FirstChild();
		if ( childNode )
		{
			PersistentTypeMap::iterator it = typeMap.find(key);
			if ( it != typeMap.end() )
			{
				PersistentData* data = &it->second;
				switch ( data->m_type )
				{
					CASE(int,  atoi( childNode->Value() ) );

					CASE2(PT_string, std::string, childNode->Value() );

					CASE(bool, (_stricmp( childNode->Value(), "true" ) == 0) ? true : false);					
				}
			}
		}

		if ( element->NextSibling() )
			Load( element->NextSibling() );
	}

}