// Copyright (c) 2010 Luis Semp
// http://www.luissempe.com
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.

#pragma once

#include "StateGraphEvent.h"

class StateGraphData;
class BaseState;

///
/// StateGraph is the main part of the system, it will traverse the provided graph 
/// creating, processing and destroying states as it gets them.
///
class StateGraph
{
public:

	enum Status {
		Done,
		Processing
	};

	StateGraph(StateGraphData* data);
	~StateGraph();

	void Start(int startStateIndex);
	void Stop();

	Status Process();

	void SetNextState(int stateIndex);

	void Notify(StateGraphEvent& event);

	StateGraphData* GetData() const { return m_graphData; }
	
protected:

	int m_nextStateIndex;

	StateGraphData* m_graphData;

	BaseState* m_previousState;
	BaseState* m_currentState;

	BaseState* GetNextState();
};
